{--------------------------------------------------------}
{
{    Gerncia da Tabela de Smbolos
{
{    Autor: Oswaldo Vernet
{
{    Em 27/02/2012
{
{--------------------------------------------------------}


{--------------------------------------------------------}
{         calcula o "hash" de um identificador
{--------------------------------------------------------}

function hash (id : string) : FAIXA_HASH;
var
    i, len  : integer;
    sum     : int64;
begin
    len := length (id);
    sum := 0;

    for i := 1 to len do
        sum := (sum * 2) xor ord (id[i]);

    hash := Abs (sum) mod TAM_HASH
end;

{--------------------------------------------------------}
{      retorna o tamanho da maior lista de colises
{--------------------------------------------------------}

function maiorLista : integer;
var
    n, h, m : integer;
    p       : PTabSimb;
begin
    m := -1;

    for h := 0 to TAM_HASH - 1 do
    begin
        n := 0;
        p := tabelaHash[h];

        while p <> NIL do
        begin
            INC (n);
            p := p^.hprox
        end;

        if n > m then m := n
    end;

    maiorLista := m
end;

{--------------------------------------------------------}
{         busca uma varivel (ou funo) na tabela
{--------------------------------------------------------}

function buscaVariavel (s : integer; esc : PTabSimb; id : string) : PTabSimb;
var
    local, global, p : PTabSimb;
begin
    local := NIL; global := NIL;

    p := tabelaHash[hash (id)];       { Incio da lista de coliso }

    while p <> NIL do                 { Percorre toda a lista de colises }
    begin
        if (p^.id = id) and ((p^.script < 0) or (p^.script = s)) then
        begin
            if (p^.escopo = esc) and (esc <> NIL) then
                local := p;

            if p^. escopo = NIL then
                global := p
        end;

        p := p^.hprox
    end;

    if local = NIL then local := global;

    buscaVariavel := local
end;

{--------------------------------------------------------}
{         busca um identificador na tabela
{--------------------------------------------------------}

function buscaSimbolo (s : integer; esc : PTabSimb; id : string; install : boolean) : PTabSimb;
var
    h           : integer;
    p, q, ant   : PTabSimb;
begin
    h := hash (id);

    ant := NIL;
    p   := tabelaHash[h];       { Incio da lista de coliso }

    while p <> NIL do           { Busca na lista de colises }
    begin
        if (p^.id = id) and ((p^.tipo in [S_COM..S_END,S_NFUNC]) or (((p^.script < 0) or (p^.script = s)) and (p^.escopo = esc))) then
        begin
            buscaSimbolo := p;  { Achou! }
            exit
        end;

        ant := p;                { Colidiu: tenta o prximo }
        p := p^.hprox
    end;

    q := NIL;                    { Prepara o retorno }

    if install then              { O smbolo  novo e deve ser instalado }
    begin
        try NEW (q) except end;
        
        if q <> NIL then
        begin
            q^.id       := id;
            q^.script   := s;
            q^.escopo   := esc;
            q^.prox     := NIL;
            q^.tipo     := S_INDEF;     { Inicialmente no sabemos o tipo }

            q^.val.iter     := -1;
            q^.val.exp.tipo := E_INVAL;     { ... tampouco o valor }

            q^.hprox    := p;           { Insere na lista de colises }
            q^.hprev    := ant;

            if p <> NIL then
                p^.hprev := q;

            if ant = NIL then
                tabelaHash[h] := q
            else
                ant^.hprox := q;

            ultSimb^.prox := q;         { Insere tambm na lista de smbolos }
            ultSimb := q
        end
        else begin
            mensagemDeErro ('Memria insuficiente')
        end
    end;

    buscaSimbolo := q
end;

{--------------------------------------------------------}
{         cria uma varivel local (ou parmetro)
{--------------------------------------------------------}

function criaVariavelLocal (esc : PTabSimb; id : string; desloc : integer) : PTabSimb;
var
    p : PTabSimb;
begin
    p := buscaSimbolo (scriptCorrente, esc, id, true);

    if not (p^.tipo in [S_VAR,S_INDEF]) then
    begin
        mensagemDeErro ('"' + id + '"  identificador de ' + tipoSimbParaCadeia[p^.tipo] + ', no pode ser uma varivel local ou parmetro');
        criaVariavelLocal := NIL;
        exit
    end;

    p^.tipo   := S_VAR;
    p^.desloc := desloc;

    criaVariavelLocal := p;
end;

{--------------------------------------------------------}
{         instala um smbolo fixo na tabela hash
{--------------------------------------------------------}

procedure instalaSimboloFixo (q : PTabSimb);
var
    p : ^PTabSimb;      { Ponteiro duplo! }
begin
    { Smbolos fixos so inseridos apenas na tabela hash.
      O n j vem pronto, no  preciso criar! }

    p  := @tabelaHash[hash (q^.id)];

    q^.script := -1;     { So independentes de script }
    q^.escopo := NIL;
    q^.prox   := NIL;
    q^.hprox  := p^;     { Insere no incio da lista de coliso }
    q^.hprev  := NIL;

    if p^ <> NIL then
        p^^.hprev := q;

    p^        := q
end;

{--------------------------------------------------------}
{             instala um rtulo na tabela
{--------------------------------------------------------}

function instalaRotulo (s : integer; esc : PTabSimb; id : string; linha : integer) : boolean;
var
    p   : PTabSimb;
begin
    instalaRotulo := false;

    p := buscaSimbolo (s, esc, id, true);

    if p = NIL then exit;

    if p^.tipo = S_INDEF then
    begin
        p^.tipo       := S_ROT;         { Define o rtulo }
        p^.linha      := linha;
        instalaRotulo := true
    end
    else if p^.tipo = S_ROT then
    begin
        mensagemDeErro (linha, 'Rtulo "' + id + '" j foi definido na linha ' + intToStr (p^.linha))
    end
end;

{--------------------------------------------------------}
{             busca um rtulo no script
{--------------------------------------------------------}

function buscaRotulo (s : integer; esc : PTabSimb; rotulo: string): integer;
var
    p   : PTabSimb;
begin
    if (length (rotulo) = 0) or (rotulo[1] <> '@') then
        rotulo := '@' + rotulo;

    p := buscaSimbolo (s, esc, maiuscAnsi (rotulo), false);

    if (p = NIL) or (p^.tipo <> S_ROT) then
        buscaRotulo := -1
    else
        buscaRotulo := p^.linha
end;

{--------------------------------------------------------}
{             busca um comando
{--------------------------------------------------------}

function buscaComando (id : string) : PTabSimb;
var
    p : PTabSimb;
begin
    p := tabelaHash[hash (id)]; { Incio da lista de coliso }

    while p <> NIL do           { Busca na lista de colises }
    begin
        if (p^.id = id) and (p^.tipo in [S_COM..S_END]) then
            break;

        p := p^.hprox
    end;

    buscaComando := p
end;

{--------------------------------------------------------}
{        apaga os smbolos locais a uma funo
{--------------------------------------------------------}

function apagaSimbolosLocais (s : integer; esc : PTabSimb; tipos : SetTipoSimb) : boolean;
var
    ant, p  : PTabSimb;
    h       : integer;
begin
    if (esc <> NIL) and (esc^.script <> s) then
    begin
        apagaSimbolosLocais := false;
        exit
    end;

    ant := @primSimb; p := ant^.prox;

    while p <> NIL do
    begin
        if (p^.tipo in tipos) and (p^.script = s) and (p^.escopo = esc) then
        begin
            h := hash (p^.id);

            if p^.hprev = NIL then
                tabelaHash[h] := p^.prox
            else
                p^.hprev^.hprox := p^.hprox;

            if p^.hprox <> NIL then
                p^.hprox^.hprev := p^.hprev;

            ant^.prox := p^.prox;
            p := ant^.prox
        end
        else begin
            ant := p;
            p := p^.prox
        end
    end;

    apagaSimbolosLocais := true
end;

{--------------------------------------------------------}
{        apaga os smbolos globais de um script
{--------------------------------------------------------}

procedure apagaSimbolos (s : integer);
var
    ant, p  : PTabSimb;
    h       : integer;
begin
    ant := @primSimb; p := ant^.prox;

    while p <> NIL do
    begin
        if p^.script = s then
        begin
            if p^.tipo = S_VAR then
            begin
                if (p^.escopo = NIL) and (p^.desloc = 0) then
                    liberaExpr (p^.val.exp)
            end;

            h := hash (p^.id);

            if p^.hprev = NIL then
                tabelaHash[h] := p^.prox
            else
                p^.hprev^.hprox := p^.hprox;

            if p^.hprox <> NIL then
                p^.hprox^.hprev := p^.hprev;

            ant^.prox := p^.prox;
            p := ant^.prox
        end
        else begin
            ant := p;
            p := p^.prox
        end
    end
end;

{--------------------------------------------------------}
{               inicializa a tabela hash
{--------------------------------------------------------}

procedure inicializaTabelaHash;
var
    i : integer;
begin
    for i := 0 to TAM_HASH - 1 do
        tabelaHash[i] := NIL
end;

{--------------------------------------------------------}
{             inicializa a tabela de smbolos
{--------------------------------------------------------}

procedure inicializaTabelaDeSimbolos;
begin
    primSimb.prox := NIL;           { O primeiro n no contm informao util }
    ultSimb       := @primSimb
end;




