{--------------------------------------------------------}
{
{    DVSCRIPT.PAS - UNIT principal do ScriptVox
{
{    Autor: Jos Antonio Borges
{
{    Em 23/06/2000
{
{    Modificado por: Oswaldo Vernet
{
{    Em: 27/02/2012
{
{--------------------------------------------------------}

unit DVSCRIPT;

{--------------------------------------------------------}
{                    I N T E R F A C E
{--------------------------------------------------------}

interface

uses
    dvwin, dvcrt, windows, classes, messages, sysUtils, mmsystem,
    dvmacro, dvexec, winsock, dvinet, dvmsaa, dvform, dvjpeg, interp;

type
    RESULTADO_SCRIPT = ( SCR_OK, SCR_ERROEXEC, SCR_SEMARQUIVO, SCR_ROTULOINVALIDO, SCR_ERROSINTAXE );

procedure setaInterativo (b : boolean);
procedure zeraVarScript;

function  executaScript (nomeArq, rotulo: string; var ultLinha: integer; var mudo : boolean): RESULTADO_SCRIPT; overload;
function  executaScript (nomeArq, rotulo: string; var ultLinha: integer): RESULTADO_SCRIPT; overload;
function  executaScriptControlador (nomeArq : string; rotina : RotinaExterna; var ultLinha: integer) : RESULTADO_SCRIPT;

function  cmdScript (cmd : string; var terminou : boolean; var mudo : boolean) : boolean; overload;
function  cmdScript (cmd : string) : boolean; overload;

function  obtemVarLongaScript (nome: string): string;
procedure alteraVarLongaScript (nome, valor : string);

{--------------------------------------------------------}
{               I M P L E M E N T A   O
{--------------------------------------------------------}

implementation

{--------------------------------------------------------}
{      executa uma linha em processamento externo
{--------------------------------------------------------}

function cmdScript (cmd : string; var terminou : boolean; var mudo : boolean) : boolean; overload;
begin
    try
        cmdScript     := executaLinhaAvulsa (cmd, terminou, mudo)
    except
        on e : Exception do
        begin
            mudo      := false;
            terminou  := true;
            cmdScript := true;

            if not mudo then
                sintWriteln (e.Message)
        end
    end
end;

function cmdScript (cmd : string) : boolean; overload;
var
    terminou, mudo : boolean;
begin
    cmdScript := cmdScript (cmd, terminou, mudo)
end;

{--------------------------------------------------------}
{                   executa um script
{--------------------------------------------------------}

function executaScript (nomeArq, rotulo: string; var ultLinha: integer; var mudo : boolean) : RESULTADO_SCRIPT; overload;
var
    linha   : integer;
    s       : integer;
begin
    ultLinha      := 1;

    s := carregaScript (nomeArq);

    if s < 0 then
    begin
        executaScript := SCR_SEMARQUIVO;
        exit
    end;

    if not preCompila (s) then
    begin
        ultLinha      := PC (s);
        executaScript := SCR_ERROSINTAXE;
        exit
    end;

    if (rotulo <> '') then     { Foi dado um rtulo como Ponto de Entrada }
    begin
        linha := achaRotulo (s, rotulo);

        if linha <= 0 then
        begin
            executaScript := SCR_ROTULOINVALIDO;
            exit
        end
    end
    else begin
        linha := 0
    end;

    mudaScriptCorrente (s);

    try
        if executa (s, linha + 1, false) then
            executaScript := SCR_OK
        else
            executaScript := SCR_ERROEXEC
    except
        on e : Exception do
        begin
            if not pegaMudo (s) then
                sintWriteln (e.Message);

            executaScript := SCR_OK
        end
    end;

    ultLinha := PC (s);
    mudo     := pegaMudo (s)
end;

function executaScript (nomeArq, rotulo: string; var ultLinha: integer) : RESULTADO_SCRIPT; overload;
var
    mudo : boolean;
begin
    executaScript := executaScript (nomeArq, rotulo, ultLinha, mudo)
end;

{--------------------------------------------------------}
{      executa o script, com acesso a rotina externa
{--------------------------------------------------------}

function executaScriptControlador (nomeArq : string; rotina : RotinaExterna; var ultLinha: integer) : RESULTADO_SCRIPT;
begin
    if @rotina <> NIL then
        endRotinaExterna := rotina;

    executaScriptControlador := executaScript (nomeArq, '', ultLinha)
end;

{--------------------------------------------------------}
{         modifica o modo de execuo do script
{--------------------------------------------------------}

procedure setaInterativo (b : boolean);
begin
    interativo := b
end;

{--------------------------------------------------------}
{             obtm o valor de uma varivel
{--------------------------------------------------------}

function obtemVarLongaScript (nome: string): string;
begin
    obtemVarLongaScript := obtemVar (nome)
end;

{--------------------------------------------------------}
{            modifica o valor de uma varivel
{--------------------------------------------------------}

procedure alteraVarLongaScript (nome, valor: string);
begin
    if not modificaVar (nome, valor) then
        sintWriteln ('Erro ao modificar a varivel ' + nome)
end;

{--------------------------------------------------------}
{     inicializa variveis globais do interpretador
{--------------------------------------------------------}

procedure zeraVarScript;
begin
    inicializaInterpretador
end;

begin
    zeraVarScript
end.


