{--------------------------------------------------------}
{
{      Processador de Scripts
{
{      Autor: Jos Antonio Borges
{                                                    
{      Em 25/6/2000                                            
{
{      Reescrito por: Oswaldo Vernet
{                                                                                      
{      Em 27/02/2012
{
{--------------------------------------------------------}

program sc;

uses
  windows,
  classes,
  sysUtils,
  dvwin,
  dvcrt,
  dvarq,
  dvform,
  dvmsaa,
  activeX,
  dvscript,
  INTERP in 'interpretador\INTERP.PAS';

const
    versao = '6.0';

var
    nomeArq, rotulo : string;
    modoInterativo  : boolean;

{--------------------------------------------------------}
{                 fala uma mensagem
{--------------------------------------------------------}

procedure mensagem (nomeArq: string; apular: integer);
var
    i   : integer;
    s   : string;
begin
    if      nomeArq = 'SCINIC'      then  s := 'ScriptVox - v'
    else if nomeArq = 'SCNOMARQ'    then  s := 'Nome do arquivo .PRO a executar: '
    else if nomeArq = 'SCFIM'       then  s := 'Fim do script'
    else if nomeArq = 'SCARQNAO'    then  s := 'Arquivo .PRO inexistente'
    else if nomeArq = 'SCERREXE'    then  s := 'Erro de execuo na linha '
    else if nomeArq = 'SCERRROT'    then  s := 'Este rtulo no existe'
    else if nomeArq = 'SCRERROSINT' then  s := 'Script com erro sinttico na linha '
    else if nomeArq = 'SCERRO'      then  s := 'Erro'
    else if nomeArq = 'SCEDITE'     then  s := 'Editore e aperte enter'
    else if nomeArq = 'SCINTER'     then  s := 'Modo interativo'
    else begin
        writeln;
        writeln ('-------------------------------');
        writeln ('--- mensagem errada: ', s, '---');
        writeln ('-------------------------------');
        writeln;
    end;

    write (s);

    for i := 1 to apular do writeln;

    if existeArqSom (nomeArq) then
        sintsom (nomeArq)
    else
        sintetiza (s)
end;

{--------------------------------------------------------}
{             transforma cadeia em maiusculos
{--------------------------------------------------------}

function maiusc (s: string): string;
begin
    s := s + #$0;
    ansiUpper (@s[1]);
    delete (s, length (s), 1);
    maiusc := s;
end;

{--------------------------------------------------------}
{                    inicializao
{--------------------------------------------------------}

function inicializa: boolean;
label
    volta;
var
    s           : string;
    dirAtual    : string;
    p           : integer;
begin
    CoInitialize (NIL);
    inicializa := true;

    sintInic (0, sintAmbiente ('SCRIPVOX', 'DIRSCRIPVOX'));
    clrscr;
    setWindowText (crtWindow, 'SCRIPVOX');

    if paramCount <> 0 then
    begin
        nomeArq := paramStr(1)
    end
    else begin
        textBackground (BLUE);
        mensagem ('SCINIC', 0);   {'ScriptVox - v'}
        sintWriteln (versao);
        textBackground (BLACK);
        writeln;

        getDir (0, dirAtual);

        s := sintAmbiente ('SCRIPVOX', 'DIRDEFAULT');

        if s = '' then s := 'c:\winvox\scripts';
{$I-}
        ChDir (s);
{$I+}
        if ioresult <> 0 then ;

        mensagem ('SCNOMARQ', 0);    {'Nome do arquivo .PRO a executar: '}

        nomeArq := trim (obtemNomeArq (20));
        writeln (nomeArq);

        if teclaObtemNomeArq = ESC then
        begin
            sintFim;
            doneWinCrt
        end;

        ChDir (dirAtual)
    end;

    modoInterativo := false;
    if nomeArq = '' then
    begin
        s := 'SCRIPVOX (interativo)';
        setWindowText (crtWindow, @s[1]);
        modoInterativo := true;
        exit
    end;

    rotulo := '';
    p := pos ('@', nomeArq);

    if p <> 0 then
    begin
        rotulo := trim (copy (nomeArq, p + 1, 999));
        delete (nomeArq, p, 999);
        nomeArq := trim (nomeArq)
    end;

    s := 'SCRIPVOX ' + nomeArq;
    setWindowText (crtWindow, @s[1]);

volta:
    if fileExists (nomeArq) then exit;

    if pos ('.', nomeArq) = 0 then
    begin
        if fileExists (nomeArq + '.PRO') then
        begin
            nomeArq := nomeArq + '.PRO';
            exit
        end
    end;

    if pos ('\', nomeArq) = 0 then
    begin
        s := sintAmbiente ('SCRIPVOX', 'DIRDEFAULT');

        if s <> '' then
        begin
            if s [length(s)] <> '\' then
                s := s + '\';

            nomeArq := s + nomeArq;
            goto volta
        end
    end;

    mensagem ('SCARQNAO', 1);   {'Arquivo .PRO inexistente'}
    inicializa := false
end;

{--------------------------------------------------------}
{                    processamento
{--------------------------------------------------------}

procedure processaInterativo;
label
    fim;
var
    c               : char;
    i               : integer;
    listaCmd        : TStringList;
    linha           : string;
    terminou, mudo  : boolean;
begin
    mensagem ('SCINTER', 2);    {'Modo interativo'}

    listaCmd  := TStringList.create;
    mudo      := false;
    terminou  := false;

    while not terminou do
    begin
        linha := '';

        repeat
            c := sintEdita (linha, wherex, wherey, 255, true);
            writeln;

            if c = ESC then
                goto fim;

            if (c = CIMA) or (c = BAIX) then
            begin
                garanteEspacoTela (5);
                popupMenuCria (wherex, wherey, 80, listaCmd.Count, RED);

                for i := 0 to listaCmd.Count-1 do
                   popupMenuAdiciona ('', listaCmd[i]);

                i := popupMenuSeleciona;
                if i >= 1 then
                begin
                    linha := opcoesItemSelecionado;
                    mensagem ('SCEDITE', 1)
                end
            end
        until (c = ENTER) or (c = ESC);

        if listaCmd.Count = 0 then
            listaCmd.Add (linha)
        else
            listaCmd.Insert (0, linha);

        if not cmdScript (linha, terminou, mudo) then
            mensagem ('SCERRO', 1);   {'Erro'}

        linha := ''
    end;

fim:
    if not mudo then
        mensagem ('SCFIM', 1)   {'Fim do script'}
end;

{--------------------------------------------------------}
{                    processamento
{--------------------------------------------------------}

procedure processa;
var
    ultLinProc  : integer;
    mudo        : boolean;
begin
    case executaScript (nomeArq, rotulo, ultLinProc, mudo) of
        SCR_OK:
            if not mudo then
                mensagem ('SCFIM', 1);   {'Fim do script'}

        SCR_ERROEXEC:
            begin
                showWindow (crtWindow, SW_RESTORE);
                mensagem ('SCERREXE', 0);   {'Erro de execuo na linha '}
                sintWriteint (ultLinProc);
                writeln;
                readln;
            end;

        SCR_SEMARQUIVO:
            mensagem ('SCARQNAO', 1);   {'Arquivo inexistente'}

        SCR_ROTULOINVALIDO:
            mensagem ('SCERRROT', 1);   {''Este rtulo no existe !'}

        SCR_ERROSINTAXE:
            begin
                mensagem ('SCRERROSINT', 0);
                sintWriteint (ultLinProc);
                writeln;
                readln
            end
    end
end;

{--------------------------------------------------------}
{                      finalizao
{--------------------------------------------------------}

procedure finaliza;
begin
    CoUnInitialize;
    sintFim;
    doneWinCrt
end;

{--------------------------------------------------------}
{     Programa Principal do Interpretador ScriptVox
{--------------------------------------------------------}

begin
    if inicializa then
    begin
        setaInterativo (modoInterativo);

        if modoInterativo then
            processaInterativo
        else
            processa
    end
    else begin
        delay (2000)
    end;

    finaliza
end.
